import { Component, EventEmitter, Input, OnChanges, OnInit, Output, SimpleChanges } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { forkJoin, lastValueFrom } from 'rxjs';
import { ButtonType } from '../../shared/components/button/buttonType.enum';
import { SearchService } from '../../structure-list/services/search.service';
import { TypeModal } from '../enums/TypeModal.enum';
import { SearchQuery } from '../models/searchQuery.model';

@Component({
  selector: 'app-search-bar',
  templateUrl: './search-bar.component.html',
  styleUrls: ['./search-bar.component.scss'],
})
export class SearchBarComponent implements OnInit, OnChanges {
  @Input() shouldResetFilters = 0;
  @Input() shouldShowMore = 0;
  @Output() searchEvent = new EventEmitter<SearchQuery>();
  @Output() shouldLoad = new EventEmitter<boolean>();

  public addStructureFormModal = false;
  public buttonTypeEnum = ButtonType;
  public searchForm: FormGroup;
  public modalTypeOpened: TypeModal;
  public employersFiltersActive = false;
  public jobsFiltersActive = false;
  public jobTypes: string[] = [];
  public employerTypes: string[] = [];
  public TypeModal = TypeModal;

  public jobFilterChecked: string[] = [];
  public employerFilterChecked: string[] = [];

  constructor(
    private fb: FormBuilder,
    private activatedRoute: ActivatedRoute,
    private route: ActivatedRoute,
    private router: Router,
    public searchService: SearchService
  ) {}

  async ngOnInit(): Promise<void> {
    // Will store the different categories
    await this.loadFilters();
    const queryString = this.activatedRoute.snapshot.queryParamMap.get('search');
    // Use existing query if back to the page or init the query
    if (!this.searchService.annuaireSearchQuery) {
      this.searchService.annuaireSearchQuery = { queryParam: '', page: 1, jobFilters: [], employerFilters: [] };
    }
    if (queryString) {
      this.searchService.annuaireSearchQuery.queryParam = queryString;
    }
    this.splitFilters(this.searchService.checkedFilterList);
    this.searchForm = this.fb.group({
      searchTerm: this.searchService.annuaireSearchQuery.queryParam,
    });
    const term = this.searchForm.get('searchTerm').value || '';
    this.searchEvent.emit({
      queryParam: term,
      jobFilters: this.jobFilterChecked,
      employerFilters: this.employerFilterChecked,
    });
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (changes.shouldResetFilters && changes.shouldResetFilters.currentValue !== 0) {
      this.resetFilters();
      this.shouldLoad.emit(true);
    }
    if (changes.shouldShowMore && !changes.shouldShowMore.firstChange) {
      const term = this.searchForm.get('searchTerm').value || '';
      this.searchEvent.emit({
        queryParam: term,
        page: changes.shouldShowMore.currentValue,
        jobFilters: this.jobFilterChecked,
        employerFilters: this.employerFilterChecked,
      });
    }
  }

  /** Get the categories for each modal type */
  private async loadFilters(): Promise<void> {
    const { job, employer } = await lastValueFrom(
      forkJoin({
        job: this.searchService.getJobsGroups(),
        employer: this.searchService.getEmployers(),
      })
    );
    this.jobTypes = job;
    this.employerTypes = employer;
    this.countCheckedFilters();
  }

  public clearInput(): void {
    this.searchForm.reset();
    this.applyFilter('');
  }

  /** Sends an array containing all filters */
  public applyFilter(term: string): void {
    this.shouldLoad.emit(true);
    this.shouldResetFilters = 0;
    // Add search input filter
    if (term) {
      this.router.navigate(['/annuaire'], {
        relativeTo: this.route,
        queryParams: {
          search: term,
        },
        queryParamsHandling: 'merge',
      });
    } else if (!term) {
      this.router.navigate(['/annuaire'], {
        relativeTo: this.route,
      });
    }
    this.splitFilters(this.searchService.checkedFilterList);
    this.searchEvent.emit({
      queryParam: term || '',
      jobFilters: this.jobFilterChecked,
      employerFilters: this.employerFilterChecked,
    });
  }

  private splitFilters(checkedFilterList: string[]): void {
    this.employerFilterChecked = checkedFilterList.filter((filter) => this.employerTypes.includes(filter));
    this.jobFilterChecked = checkedFilterList.filter((filter) => this.jobTypes.includes(filter));
  }

  public fetchResults(checkedFilters: string[]): void {
    this.searchService.checkedFilterList = checkedFilters;
    const inputTerm = this.searchForm.get('searchTerm').value;
    this.closeModal();
    this.applyFilter(inputTerm);
  }

  public getModalCategory(): string[] {
    switch (this.modalTypeOpened) {
      case TypeModal.jobs:
        return this.jobTypes;
      case TypeModal.employers:
        return this.employerTypes;
      default:
        return null;
    }
  }

  /** Open the modal and display the list according to the right filter button */
  public openModal(modalType: TypeModal): void {
    // if modal already opened, reset type
    if (this.modalTypeOpened === modalType) {
      this.closeModal();
    } else if (this.modalTypeOpened !== modalType) {
      this.modalTypeOpened = modalType;
    }
  }

  public closeModal(): void {
    this.modalTypeOpened = undefined;
    this.countCheckedFilters();
  }

  /** Check if some modules is checked on filter and store number of modules checked */
  private countCheckedFilters(): void {
    this.jobsFiltersActive = this.searchService.checkedFilterList.some((filter) => this.jobTypes.includes(filter));
    this.employersFiltersActive = this.searchService.checkedFilterList.some((filter) =>
      this.employerTypes.includes(filter)
    );
  }

  public resetFilters(): void {
    this.shouldLoad.emit(true);
    this.searchService.checkedFilterList = [];
    this.employersFiltersActive = false;
    this.jobsFiltersActive = false;
    this.employerFilterChecked = [];
    this.jobFilterChecked = [];
    this.searchEvent.emit({ queryParam: '', jobFilters: [], employerFilters: [] });
    this.router.navigate(['/annuaire']);
    this.searchForm.reset();
  }

  public removeFilter(filter: string): void {
    this.shouldLoad.emit(true);
    const index = this.searchService.checkedFilterList.findIndex((checkedFilter: string) => checkedFilter === filter);
    this.searchService.checkedFilterList.splice(index, 1);
    const inputTerm = this.searchForm.get('searchTerm').value;
    this.splitFilters(this.searchService.checkedFilterList);
    this.countCheckedFilters();
    this.searchEvent.emit({
      queryParam: inputTerm || '',
      jobFilters: this.jobFilterChecked,
      employerFilters: this.employerFilterChecked,
    });
  }
}
